// Display a sprite and do many things with the GsSprite Structure 
// This is a combined file with pad routines from Nelson Santos/ROGUE
//	and Sprite.c from Ira Rainey
//
// I Like this.. Its got funky rotational velocity a neat copy and a dabble with ordering tables.
// Well ok not very much to get excited about, but these are the roots.
// "Using these simple foundations, imaginaton becomes reality." - Confused Beard 1998
//
// Triangle, Circle & Cross change RGB Values
// 	Square to reset 
//
// Now with added:
// 	Velocity to rotation (R1 & L1)
// 	Velocity to X,Y
// 	Select on Pad1 to copy the current sprite
// 	R2 & L2 To move up & down the ordering table
//
// To see the R2 & L2 at work first press select to copy the sprite then press
// 	R2 / L2 and notice how the current sprite goes behind and in front of the orginal!!
//
// Techy stuff..
// 	A new structure called Sprite which holds the velocties
// 	More sprites could be displayed using the "new improved" recoded sprite arrays



// -----------------------------------------------
// Includes
// -----------------------------------------------

#include <libps.h>
#include "rougepad.h"				// Include the rougepad.h header file


// -----------------------------------------------
// Defines
// -----------------------------------------------

// Total number of sprites plus one
#define		SPRITE_CNT	(2+1)

// Screen size detail
#define		SCREEN_WIDTH	320
#define		SCREEN_HEIGHT	240

// Tim address
// Load your TIM file here
#define		spriteAdd	0x80100000

// Set ordering table length 	4 = 0 to 15 z depths
//						8 = 0 to 255 z depths
// If you put an object outside of these ranges.... Kaboom
#define OT_LENGTH	8

// Velocity
#define VEL 2


// -----------------------------------------------
// Function Prototypes
// -----------------------------------------------
void InitSprite ();
void SomePad1 ();
void AICode ();

// -----------------------------------------------
// Initialise Structures
// -----------------------------------------------

// Ordering table variables & Global Vars
GsOT			WorldOT[2];
GsOT_TAG		OTTags[2][1<<OT_LENGTH];
int				activeBuff;
int Quit = 0;

// This Sprite Structure Array holds the velocity information
struct {
	int Active,RVel,XVel,YVel,ZDepth,SpriteNo;
} Sprite[SPRITE_CNT - 1];


// Primitive related vaiables
PACKET			GpuPacketArea[2][SPRITE_CNT*sizeof(GsSPRITE)];

// Sprite structure pointer
GsSPRITE		TestSprite[SPRITE_CNT-1];

void main(void)
{
	int	i;

	PadInit();					// First initialize the joypads
	SetVideoMode (MODE_PAL);


	// Initialise the drawing system
	GsInitGraph(SCREEN_WIDTH,SCREEN_HEIGHT,4,0,0);
	GsDefDispBuff(0,0,0,SCREEN_HEIGHT);
	ResetGraph(0);
		
	// Initialise the ordering table
	for (i=0;i<2;i++)
		{
		WorldOT[i].length = OT_LENGTH;
		WorldOT[i].org = OTTags[i];
		}

	// Initialise the sprite
	InitSprite();


	while (Quit == 0)
	{
		AICode();

		Pad1 = PadRead(1);		// Read in input data from Pad 1
		Pad2 = PadRead(2);		// Read in input data from Pad 2

		if (Pad1 != 0)
			{
				SomePad1();
			}


		// Find out which buffer is being used
		activeBuff = GsGetActiveBuff();

		// Packet area
		GsSetWorkBase((PACKET *)GpuPacketArea[activeBuff]);

		// Clear the ordering table
		GsClearOt(0,0,&WorldOT[activeBuff]);

  		// Register sprite into ordering table
		// Uses GsSortSprite for rotation/scaling
		for (i=0;i<(SPRITE_CNT-1);i++)
		{
			printf("%d loop = %d ZDepth = %d\n",SPRITE_CNT - 1,i,Sprite[i].ZDepth);
			
			if (Sprite[i].Active > 0)
			{
				GsSortSprite(&TestSprite[i],&WorldOT[activeBuff],Sprite[i].ZDepth);
			}
		}

			
		// Wait for all drawing to finish
		DrawSync(0);

		// Wait for vertical blank interrupt
		VSync(0);

		// Switch display buffer & offscreen buffer
		GsSwapDispBuff();

		// Sort the ordering table
		GsSortClear(0,0,0,&WorldOT[activeBuff]);

		// Draw the objects registered in the Ordering table
		GsDrawOt(&WorldOT[activeBuff]);
	}
}



//
// Sprite initialisation routine
//

void InitSprite()
	{
	RECT		rect;							// Define RECT structure
	GsIMAGE	tim_data;							// Define GsIMAGE structure
	
	// Get the TIM file info - skip 4 bytes from start of file
	// to get to info we need
	GsGetTimInfo ((u_long *)(spriteAdd+4),&tim_data);

	// This line is a total must - RESET STRUCTURES!!!!
	// Without this strange values are already there from sources not of this earth
	Sprite[0].Active=Sprite[0].RVel=Sprite[0].XVel=Sprite[0].YVel=Sprite[0].ZDepth=Sprite[0].SpriteNo=0;

	// Just A Hint On Tims:
	// 	When placing tims within the frame buffer on your tim util v2.0a
	//	If you don't place them in the top lefthand corner of the texture page you must
	//	supply the u & v offset values


	// Load the image into the frame buffer
	rect.x = tim_data.px;							// x pos in frame buffer
	rect.y = tim_data.py;							// y pos in frame buffer
	rect.w = tim_data.pw;							// width of image
	rect.h = tim_data.ph;							// height of image
	LoadImage(&rect, tim_data.pixel);				// load data into frame buffer

	// Load the CLUT into the frame buffer
	rect.x = tim_data.cx;							// x pos in frame buffer
	rect.y = tim_data.cy;							// y pos in frame buffer
	rect.w = tim_data.cw;							// width of CLUT
	rect.h = tim_data.ch;							// height of CLUT
	LoadImage(&rect, tim_data.clut);				// load data into frame buffer

	// Initialise sprite structure
	TestSprite[0].attribute = 0x01000000;						// sprite attribute - 8 bit
	TestSprite[0].x = ((SCREEN_WIDTH/2)-(tim_data.pw));		// sprite x axis position
	TestSprite[0].y = ((SCREEN_HEIGHT/2)-(tim_data.ph/2));// sprite y axis position
	
	// The width of an 8 bit sprite is half that of a 16 bit hence the *2
	TestSprite[0].w = (tim_data.pw*2);				// sprite width
	TestSprite[0].h = tim_data.ph;					// sprite height
	TestSprite[0].tpage = GetTPage(1,0, tim_data.px, tim_data.py);
	TestSprite[0].u = 0;							// Texture page offset x axis
	TestSprite[0].v = 0;							// Texture page offset y axis
	TestSprite[0].cx = tim_data.cx;					// CLUT x axis position
	TestSprite[0].cy = tim_data.cy;					// CLUT y axis position
	TestSprite[0].r = 128;							// Regular brightness
	TestSprite[0].g = 128;							// Regular brightness
	TestSprite[0].b = 128;							// Regular brightness
	TestSprite[0].mx = TestSprite[0].w / 2;				// Middle of rotation x
	TestSprite[0].my = TestSprite[0].h / 2;				// Middle of rotation y
	TestSprite[0].scalex = ONE;						// Defined as 4096
	TestSprite[0].scaley = ONE;						// Defined as 4096
	TestSprite[0].rotate = 0;						// Initial rotation
	Sprite[0].ZDepth = 15;
	Sprite[0].Active = 1;
	
	// Wait until LoadImage() has finished before returning
	DrawSync(0);
	}



//
// Something happed to PAD1 Routine
//

void SomePad1()
	{
			if ( (Pad1 & PADSTART) && (Pad1 & PADSELECT) )
				{
					printf("You pressed the Quit button combo on Pad 1. Bye!\n\n");
					Quit = 1;
				}
			if (Pad1 & PADSELECT)
				{
					Sprite[1] = Sprite[0];
					TestSprite[1] = TestSprite[0];		// Piece Of cake when using arrays
					printf("Copy\n");
				}


				
			if  (Pad1 & PADRIGHT)
				{
					Sprite[0].XVel+=VEL;
				} 
			if  (Pad1 & PADLEFT)
				{
					Sprite[0].XVel-=VEL;
				} 		
			if  (Pad1 & PADDOWN)
				{
					Sprite[0].YVel+=VEL;
				} 
			if  (Pad1 & PADUP)
				{
					Sprite[0].YVel-=VEL;
				} 

			// rotate sprite clockwise
			if (Pad1 & PADL1)
				{
					Sprite[0].RVel+=ONE;
				}
	
			// rotate sprite anti-clockwise	
			if (Pad1 & PADR1)
				{
					Sprite[0].RVel-=ONE;
				}
	
			// scale sprite down in size	
			if (Pad1 & PADL2) 
				{
					Sprite[0].ZDepth-=1;
					if (Sprite[0].ZDepth<0) Sprite[0].ZDepth=0;
				}

			// scale sprite up in size
			if (Pad1 & PADR2)
				{
					Sprite[0].ZDepth+=1;
					if (Sprite[0].ZDepth>1<<OT_LENGTH) Sprite[0].ZDepth=1<<OT_LENGTH;

				}
			// Change brightness
			if (Pad1 & PADSQUARE)
				{
					TestSprite[0].r=TestSprite[0].g=TestSprite[0].b=128;
					Sprite[0].RVel=0;
					Sprite[0].YVel=0;
					TestSprite[0].x = 0;
					TestSprite[0].y = 0;
					
				}

			// Change Red
			if (Pad1 & PADCIRCLE)
				{
					TestSprite[0].r+=5;
				}
			// Change Green
			if (Pad1 & PADTRIANGLE)
				{
					TestSprite[0].g+=5;
				}

			// Change Blueness
			if (Pad1 & PADX)
				{
					TestSprite[0].b+=5;
				}
		}

// AICode handles all the motion
void AICode()
	{
		int XDamp = 1;
		int YDamp = 1;
		int Damp = (abs(Sprite[0].RVel / 30));

//		printf("RotVel %d ",Sprite[0].RVel);
		
		if (Sprite[0].RVel > Damp)
			{
				Sprite[0].RVel-=Damp;
			}

		if (Sprite[0].RVel < Damp)
			{
				Sprite[0].RVel+=Damp;
			}
		if (Sprite[0].RVel > -(50) && Sprite[0].RVel < (50))
			{
				Sprite[0].RVel = 0;
			}

		TestSprite[0].rotate+=Sprite[0].RVel;
		if (TestSprite[0].rotate>1470464) TestSprite[0].rotate=0;
		if (TestSprite[0].rotate<0) TestSprite[0].rotate=1470464;


//			 	printf(" XVel %d ",Sprite[0].XVel);
		
		if (Sprite[0].XVel > 0)
			{
				Sprite[0].XVel-=XDamp;
			}

		if (Sprite[0].XVel < 0)
			{
				Sprite[0].XVel+=XDamp;
			}

		TestSprite[0].x+=Sprite[0].XVel;

//		printf("YVel %d\n",Sprite[0].YVel);
		
		if (Sprite[0].YVel > 0)
			{
				Sprite[0].YVel-=YDamp;
			}

		if (Sprite[0].YVel < 0)
			{
				Sprite[0].YVel+=YDamp;
			}

		TestSprite[0].y+=Sprite[0].YVel;
	}
